//*** The code for the simulations of the article 
//*** "Optimization of Dose Fractionation for Radiotherapy of a Solid Tumor with Account of Oxygen Effect and Proliferative Heterogeneity"
//*** by Maxim Kuznetsov and Andrey Kolobov
//*** journal "Mathematics", Special Issue "Mathematical Modelling in Biomedicine"
//*** 2020
//*** This code works fine in Dev-C++, version 5.5.2 
//*** Note that some small parts of the code may need to be revised depending on the utilized software 

//***********************************************************************************************************************************************************
//********************************* INTRODUCTION ************************************************************************************************************
//***********************************************************************************************************************************************************

//********* Initializing

#include <stdio.h>
#include <tchar.h>
#include <windows.h>
#include <ctime>
#include <iostream>
#include <cmath>
#include <conio.h>
#include <fstream>

using namespace std;

//********* SIMULATION PARAMETERS

#define tau 0.1 // time step
#define h 0.1 // space step
#define L 1.5 // the length of a simulation region, in cm
#define N (int(100.*L/h)) // number of points in space, minus one

int SoloRun = 0; // if 1, then no optimization of fractionation scheme is performed -- only one simulation with a given schedule
#define output_frequency 24. // how often the distributions of varibles will be recorded in case of solo-simulation
#define rsn_frequency 0.1 // how often the tumor radius, speed of tumor front and number of tumor cells will be recorded in case of solo-simulation
#define inception_width_mm 0.01 // the width of inital tumor cell population, in mm
#define inception_width_int (int(10.*inception_width_mm / h)) // the number of points in space of inital tumor cell population, in cm
double RadToStartRT = 10.; // at what radius of tumor radiotherapy will begin (in mm)

//********* MODEL PARAMETERS

//********* cells & necrotic tissue

double B = 0.005; // tumor cells proliferation rate
double M = 0.01; // normal cells death rate parameter
double eps = 0.7; // ratio of death rates of tumor and normal cells due to the lack of oxygen
double Dn = 0.001; // tumor cells motility

//********* glucose

double Pg = 10.; // glucose inflow parameter
double Qgn = 6.; // tumor cells glucose consumption rate
double Qgh = 0.3; // normal cells glucose consumption rate
double gcr = 0.007; // Michaelis constant for glucose consumption rate
double Dg = 100.; // glucose diffusion coefficient

//********* oxygen

double Pw = 35.8; // oxygen inflow parameter
double wA = 5.87; // oxygen concentration in artery
double wh = 1.56; // oxygen concentration, at which hemoglobin saturation is 50%
double chi = 2.55; // Hill coefficient for oxygen-hemoglobin dissociation curve
double Qwn = 31.5; // tumor cells oxygen consumption rate
double Qwh = 8.; // normal cells oxygen consumption rate
double wcr = 0.005; // Michaelis constant for oxygen consumption rate
double Dw = 720.; // oxygen diffusion coefficient

//********* radiotherapy

double alpha = 0.1; // tumor cells linear radiosensitivity parameter
double beta = 0.01; // tumor cells quadratic radiosensitivity parameter
double OERaM = 2.5; // maximum OER_a under aerobic conditions
double OERbM = 3.; // maximum OER_b under aerobic conditions
double Km = 0.193; // Michaelis constant for oxygen enhancement effect, 3.28 mmHg
double k = 0.5; // ratio of radiosensivity of quiescent and proliferating tumor cells

//********* optimization procedure

double btoa = (1. / 3.); // ratio of normal tissue radiosensitivity parameters 
						 // note that it is inverse to classically used value for convenience -- setting it to 0. imposes limitation on the sum of fraction doses
double Dmax = 5.; // maximum fractional dose
double dS = 0.2; // the amount of radiation dose added to each fraction during the search for the "gradient"
double dD = 4.; // the coefficient fractions alteration during the "descent"
double knmin = 0.001; // minimum parameter of fractions alteration during the "descent"
double kfin = 0.98; // the threshold coefficient for determining the second stage of the scheme

//********* COUNTERS

double t = 0.; // main time counter
double ti = 0.; // for output of the distributions of varibles
double tr = 0.; // for output of the tumor radius, speed of tumor front and number of tumor cells
int d = 0; // for serial numbers of the files with distributions of varibles
double t_preRT, ti_preRT, tr_preRT; // to remember the values before radiotherapy

//********* PROCEDURES AND FUNCTIONS

void Begin(); // complex of actions for the beginning, ...
void Cycle_Ending(); // ...ending of main cycle, ...
void End(); // ... and ending of the program execution

double Mfunc(double ww); // function that defines the rate of death of cells depending on the level of oxygen
double Mw[N + 1]; // used in kinetic equations for cell death

void Oxygen_Glucose_qs(); // solving the equations for nutrients in the quasi-stationary approximation
double S(double ww); // function that defines the rate of oxygen inflow depending on the level of oxygen
double SwA = pow((wA / wh), chi) / (1 + pow((wA / wh), chi)); // value of oxygen saturation in arterial blood
#define NwS 200000000
double NmdS = (0.5 / NwS);
double Sfix[NwS + 1]; // values for S(w) are precalculated for optimization purposes
double Sw[N + 1];
double wLimS = 10.*wA; // maximum oxygen value for which precalculation will be done

void TMA(double apr, double bpr, double cpr, double F[N + 1], double prog[N + 1],
	double muleft, double hileft, double muright, double hiright); // tridiagonal matrix with algorithm constant b
void TMA_variable(double apr, double* bpr, double cpr, double F[N + 1], double prog[N + 1],
	double muleft, double hileft, double muright, double hiright); // tridiagonal matrix algorithm with variable b

void Migration_cells(); // solving the equation for tumor cells migration
void Convection(); // solving the convection equations
void ConvSpeed(double* nn, double* gg, double* FF, double* II); // computing convective speed

void Euler(); // Euler method
double Growthn(double nn, double gg, double mw); // kinetic terms for tumor cells
double Growthm(double nn, double mm, double mw); // kinetic terms for necrotic tissue

void RT(); // Radiotherapy

void Profile_output(); // output of distributions of variables

void Radius_estimation(); // estimation of tumor radius
void Radius_Speed_estimation(); // estimation of tumor radius and growth speed
void CellNumber_estimation();// estimation of number of cells
void Radius_CellNumber_Speed_O2_estimation(); // estimation of tumor radius, cell number, growth speed and O2 level
void Radius_Speed_CellNumber_output(); // output of the tumor radius, speed of tumor front and number of tumor cells

void Schedule_EF_output(); // writing down current schedule within checking uniform fractionated schemes (step 1)
void Schedule_output(); // writing down current schedule within a search for gradient (step 2)
void OptSchedule_output(); // writing down optimized schedule while going down the gradient (step 3)
void Schedule_S4_output(); // writing down optimized schedule during step 4

void SimulationRun(); // making a simulation with the current schedule
void NormalizeRTscheme(); // normalizing a scheme, accounting for maximum single dose

//********* MAIN MODEL VARIABLES

double n0[N + 1], nr[N + 1], N0[N + 1], NR[N + 1], ND[N + 1], nd[N + 1], n1[N + 1], NDC[N + 1], N1C[N + 1]; // tumor cells
double m0[N + 1], mr[N + 1], m1[N + 1], MRC[N + 1], M1C[N + 1]; // necrotic tissue
double g0[N + 1], G0[N + 1], G1[N + 1], g1[N + 1]; // glucose
double w0[N + 1], W0[N + 1], W1[N + 1], w1[N + 1]; // oxygen
double h1[N + 1]; // normal cells
double n0preRT[N + 1], m0preRT[N + 1], g0preRT[N + 1], w0preRT[N + 1]; // to remember distibutions right before the treatment

//********* AUXILIARY VARIABLES

////// for tumor cells migration

double kn = 2 * h*h / (tau*Dn);
double aprn = 1.;
double bprn = 2. + kn;
double cprn = 1.;
double Fn[N + 1];
double muleftn, hileftn, murightn, hirightn;
double progn[N + 1];
double pp[N + 1], qq[N + 1];

////// for nutrients quasistatinary solution

double alpha_g[N + 1], beta_g[N + 1];
double alpha_w[N + 1], beta_w[N + 1];
double muleftg, hileftg, murightg, hirightg;
double muleftw, hileftw, murightw, hirightw;
double norm_g, norm_w;
double norm_min = 1.e-6; // allowed error during iterative solution
double aprg = 1.;
double cprg = 1.;
double aprw = 1.;
double bprg[N + 1], bprw[N + 1];
double cprw = 1.;
double Fg[N + 1];
double progg[N + 1];
double Fw[N + 1];
double progw[N + 1];
double kg = Dg / (h*h);
double kw = Dw / (h*h);
int kqs;
int k_crit = 1000; // maximum number of iterations

////// for tumor cells convection

double F0[N + 2];
double I0[N + 2], I0_not_corr[N + 2];
double Q_plus[N + 1], Q_minus[N + 1];
int ConvRad;
double muleftm, hileftm, murightm, hirightm;
double progm[N + 1];

////// for tumor radius, speed of tumor front and number of tumor cells

double nR = 0.1; // the "detectable" amount of tumor tissue
double Rad; // current tumor radius, in mm
double Rad_prev; // previously measured tumor radius, in mm
double T_prev; // time of previous measurement of tumor radius
double Rad_preRT; // to remember value before radiotherapy

double V; // tumor front speed
double NTC; // number of tumor cells
double NPTC; // number of proliferating tumor cells
double nTC; // auxiliary variable for counting tumor cells
double O2; // average oxygen level within living tumor region
double NTC_preRT; // to remember number of tumor cells before radiotherapy

double NTCmin; // minimum number of tumor cells achieved during calculations

FILE *results; // the relevant data will be recorded in corresponding file during solo-simulation
int nopo = 0; // if equal to 1, distribution of variables will not be saved during solo-simulation

////// for work with files

char File_name1[80], num[10];
char File_name_sch[80], number[10];

////// for radiotherapy

int sizeRT = 0; // how many fractions are in the scheme
FILE * inputRT; // file with the initial RT schedule, which can be set instead of step 1, it needs to be saved in the directory with the compiled exe-file with the name "InitialScheme.txt"
double **RTscheme; // current RT schedule
double **RTscheme_opt; // current optimal RT schedule
double ***RTeff_dose; // effective dose, accounting for oxygen enhancement effect and proliferative states
double RTsurv[N + 1]; // how many tumor cells survive at each point in space
int doseRTcnt = 1; // what fraction is to be done now
int RTisOn = 0; // if it is 1, then tumor has reached the radius, at which radiotherapy starts 

////// for radiotherapy schedule optimization

double NTD = 0.; // total relative damage to tissue 
double NTDMax = 0.; // maximum total relative damage to tissue 

double NTCminInit = 0.; // minimum number of tumor cells with the initial schedule
double NTCminOpt = 0.; // minimum number of tumor cells while going down the gradient
double *NTCminArray; // will remember the minimum number of tumor cells under various schemes of radiotherapy

int ef_sch_cnt = 0; // counter of schedules during checking uniform fractionated schemes (step 1)
int EqualDoseCheckStop = 0; // flag for checking schedules with equal fractions
double EqualDose = 0.; // for remembering that fractions
int ef_best = 0; // for remembering the index number of the best uniform fractionated scheme

int grad_cnt = 0; // main counter for steps 2-4
int sch_cnt = 0; // counter of schedules during the search for gradient (step 2)
int StopS3 = 0; // flag that the descent step 3 is finished
int Nopt = 0; // counting the moves down the gradient (step 3)
double Kn = 1.; // auxiliary variable which is diminished if the step is too big

int nNorm = 0; // counter for normalization of a schedule
int nNormMax = 100000; // maximum number of attempts to normalize a schedule
double PossibleDamageError = 1.e-10; // allowed error for normalization procedure
double dr = 0.; // auxiliary variable for normalization of a schedule
double NTDbeg; // tissue damage during first stage (for step 4)
int isec; // number of the first fraction of the second stage (for step 4)
int sizeRTsaved; // to remember the number of fractions during step 4

////// miscellanea

double winit; // for checking that computatinoal size is sufficiently big
streamsize ss = cout.precision(); // for proper output on the screen
FILE *results_sch; // for output of results of a solo-simulation

//***********************************************************************************************************************************************************
//********************************* MAIN CYCLE **************************************************************************************************************
//***********************************************************************************************************************************************************

int main()
{
	Begin();
	
	if (SoloRun == 1) // only one simulation with a given schedule, it needs to be saved in the directory with the compiled exe-file with the name "InitialScheme.txt"
	{
		cout << "Schedule optimization will not be performed\n\n";

		//********* creating file for tumor radius, front speed and cell number
		if ((results = fopen("results.txt", "wt")) == NULL)
			fprintf(stderr, "Cannot open file results.txt\n\n");

		fprintf(results, "time(h)\trad(mm)\tspeed\tcells\t\tdivide\t\t<O2>\t\tO2[0]\n\n");
		fflush(results);

		// the program runs until the last irradiation is performed -- minimal number of cells is achieved at that moment
		while (doseRTcnt <= sizeRT)
		{
			t = t + tau;

			if (RTisOn == 1 && t - t_preRT - RTscheme[0][doseRTcnt] >= 0 && doseRTcnt <= sizeRT)
			{
				RT();
				doseRTcnt++;
			}

			if (t >= ti - 0.00001) // a trifle is subtracted to overcome the inevitable minor machine errors
				Profile_output();

			Euler();

			if (Dn == 0.)
				for (int i = 0; i <= N; i++)
					nd[i] = nr[i];
			else
				Migration_cells();

			Convection();
			Oxygen_Glucose_qs();

			Cycle_Ending();
			if (t >= tr)
			{
				Radius_CellNumber_Speed_O2_estimation();
				Radius_Speed_CellNumber_output();
			}

			if (RTisOn == 0 && Rad >= RadToStartRT)
			{
				RTisOn = 1;
				t_preRT = t;
				NTCmin = NTC;

				if (w0[N] < 0.99*winit)
				{
					cout << "The simulation size is not enough, oxygen concentration on the right border is " << w0[N] << ". Increase it, please\n\n";
					system("PAUSE");
					exit(0);
				}
			}
		}
		cout << "\nTime of tumor growth: " << t << " hours\n";
		cout << "\nMinimum number of tumor cells: " << NTCmin << "\n";

		//creating file for information about this schedule     	
		if ((results_sch = fopen("!Results_this_schedule.txt", "wt")) == NULL)
			fprintf(stderr, "Cannot open file !Results_this_schedule.txt\n\n");

		//writing down the information about this schedule
		fprintf(results_sch, "time(h)\tNTCmin\n\n");
		fflush(results_sch);
		fprintf(results_sch, "%.2f\t%.4e\n", t - t_preRT, NTCmin);
		fflush(results_sch);
	}

	else // optimization of radiotherapy schedule
	{
		//creating file for information about optimized schedules     	
		if ((results_sch = fopen("!Results_schedules.txt", "wt")) == NULL)
			fprintf(stderr, "Cannot open file !Results_schedules.txt\n\n");

		fprintf(results_sch, "Grad\tRun\tGain\tNTCmin\n\n");
		fflush(results_sch);

		// at first it runs up to the radius at which treatment should begin, and saves the distributions of variables 
		while (Rad < RadToStartRT)
		{
			t = t + tau;

			Euler();

			if (Dn == 0.)
				for (int i = 0; i <= N; i++)
					nd[i] = nr[i];
			else
				Migration_cells();

			Convection();
			Oxygen_Glucose_qs();

			Cycle_Ending();

			if (t >= tr)
			{
				Radius_Speed_estimation(); // we need to know speed to set endpoint
				CellNumber_estimation();
			}
		}

		cout << "Reached the point of beginning of radiotherapy. Tumor radius is " << Rad << " mm \n\n";

		// then it remembers the distributions of variables and the values of counters
		for (int i = 0; i <= N; i++)
		{
			n0preRT[i] = n0[i];
			m0preRT[i] = m0[i];
			g0preRT[i] = g0[i];
			w0preRT[i] = w0[i];
		}

		t_preRT = t;
		tr_preRT = tr;
		Rad_preRT = Rad;
		NTC_preRT = NTC;
		NTCmin = NTC;

		// radiotherapy begins
		RTisOn = 1;

		if (w0[N] < 0.99*winit)
		{
			cout << "The simulation size is not enough, oxygen concentration on the right border is " << w0[N] << ". Increase it, please\n\n";
			system("PAUSE");
			exit(0);
		}

		CellNumber_estimation();
		//writing down the information about untreated growth in a file 
		fprintf(results_sch, "%i\t%i\t\t%.4e\n", 0, 0, NTCmin);
		fflush(results_sch);

		// performing the first simulation
		if (inputRT == NULL)
			cout << "Performing the first simulation with the standard schedule\n";
		else
			cout << "Performing the first simulation with the provided schedule\n";

		SimulationRun();

		// remembering the minimum tumor cell number
		NTCminArray[0] = NTCmin;
		NTCminInit = NTCmin;
		NTCminOpt = NTCmin;

		Nopt++;

		//writing down the information about this simulation in a file 
		fprintf(results_sch, "%i\tOpt-0\t0%%\t%.4e\n", 0, NTCmin);
		fflush(results_sch);

		OptSchedule_output();
		grad_cnt++;

		cout << "\nInitial schedule: minimum tumor cell number is " << NTCmin << "\n";
		
		// if there is no preassinged schedule, this part (step 1) will find the most optimal one from the uniformly fractionated, and this schedule will be further optimized
		if (inputRT == NULL)
		{
			cout << "\nStep 1: searching for the optimal scheme from the uniformly fractionated schemes\n";

			int iSize = sizeRT;
			ef_sch_cnt = 1;

			while (EqualDoseCheckStop == 0)
			{
				if (btoa == 0.)
					EqualDose = NTDMax / iSize;
				else
					EqualDose = (sqrt(1. + 4.*btoa*NTDMax / iSize) - 1.) / (2.*btoa);

				if (EqualDose <= Dmax) // will check the corresponding schedule only if the dose is less than maximum single dose
				{
					// setting counters
					t = t_preRT;
					tr = tr_preRT;
					Rad = Rad_preRT;
					doseRTcnt = 1;
					NTCmin = NTC_preRT;

					// setting distributions of variables
					for (int i = 0; i <= N; i++)
					{
						n0[i] = n0preRT[i];
						m0[i] = m0preRT[i];
						g0[i] = g0preRT[i];
						w0[i] = w0preRT[i];

						n1[i] = n0preRT[i];
						m1[i] = m0preRT[i];
						g1[i] = g0preRT[i];
						w1[i] = w0preRT[i];
					}

					// adjusting the schedule
					for (int i = 1; i <= iSize; i++)
						RTscheme[1][i] = EqualDose;

					sizeRT = iSize;

					if (iSize < sizeRT)
						for (int i = iSize + 1; i <= sizeRT; i++)
							RTscheme[1][i] = 0.;

					// showing the schedule on screen
					Schedule_EF_output();

					// running a simulation with this schedule
					SimulationRun();

					NTCminArray[ef_sch_cnt] = NTCmin;

					//writing down the information about this simulation in a file
					fprintf(results_sch, "EF\t%i\t\t%.4e\t\n", sizeRT, NTCmin);
					fflush(results_sch);

					if (NTCminArray[ef_sch_cnt] >= NTCminArray[0])
					{
						cout.precision(ss);
						cout << "Minimum tumor cell number is " << NTCminArray[ef_sch_cnt] << " (+";
						cout.precision(5);
						cout << 100.*(NTCminArray[ef_sch_cnt] - NTCminArray[0]) / NTCminArray[0] << "%)";
					}
					else
					{
						cout.precision(ss);
						cout << "Minimum tumor cell number is " << NTCminArray[ef_sch_cnt] << " (";
						cout.precision(5);
						cout << 100.*(NTCminArray[ef_sch_cnt] - NTCminArray[0]) / NTCminArray[0] << "%)";
					}

					iSize--;
					ef_sch_cnt++;
				}
				else
					EqualDoseCheckStop = 1;
			}
			
			ef_best = 1;

			for (int i = 2; i <= ef_sch_cnt - 1; i++)
				if (NTCminArray[i] < NTCminArray[ef_best])
					ef_best = i;

			//if the best uniform fractionated scheme is worse (or equally good) than the standard one, the latter will be still used as initial
			if (NTCminArray[ef_best] >= NTCminArray[0])
			{
				sizeRT = 42;
				cout << "\n\nAll the uniformly fractionated schemes are worse, than the traditional one\n";
			}
			else // otherwise, it will be replaced by the most optimal
			{
				sizeRT = 42;
				iSize = sizeRT + 1 - ef_best;
				if (btoa == 0.)
					EqualDose = NTDMax / iSize;
				else
					EqualDose = (sqrt(1. + 4.*btoa*NTDMax / iSize) - 1.) / (2.*btoa);
				cout << "\n\nThe most optimal among uniform fractionated schemes is " << sizeRT + 1 - ef_best << " x " << EqualDose << "Gy\n";

				for (int i = 1; i <= sizeRT + 1 - ef_best; i++)
					RTscheme_opt[1][i] = EqualDose;

				if (ef_best > 1)
					for (int i = sizeRT + 2 - ef_best; i <= sizeRT; i++)
						RTscheme_opt[1][i] = 0.;

				sizeRT = min(sizeRT, sizeRT + 1 - ef_best); // zero fractions will not be varied further

				NTCminArray[0] = NTCminArray[ef_best];
			}

			for (int i = 1; i <= sizeRT; i++)
				NTCminArray[i] = 0.;
		}
		OptSchedule_output();

		while (1 == 1) // steps 2-3-4 are repeated until step 4 can not provide optimization
		{
			// the gradient descent cycle (steps 2-3) runs until local optimum is found
			Kn = 1.;
			while (Kn > knmin)
			{
				// performing the search for gradient	
				cout << "\n\nStep 2: search for gradient #" << grad_cnt;
				
				for (sch_cnt = 1; sch_cnt <= sizeRT; sch_cnt++)
				{
					// setting counters
					t = t_preRT;
					tr = tr_preRT;
					Rad = Rad_preRT;
					doseRTcnt = 1;
					NTCmin = NTC_preRT;

					// setting distributions of variables
					for (int i = 0; i <= N; i++)
					{
						n0[i] = n0preRT[i];
						m0[i] = m0preRT[i];
						g0[i] = g0preRT[i];
						w0[i] = w0preRT[i];

						n1[i] = n0preRT[i];
						m1[i] = m0preRT[i];
						g1[i] = g0preRT[i];
						w1[i] = w0preRT[i];
					}

					// adjusting the schedule
					for (int i = 1; i <= sizeRT; i++)
					{
						if (i == sch_cnt)
							RTscheme[1][i] = RTscheme_opt[1][i] + dS;
						else
							RTscheme[1][i] = RTscheme_opt[1][i];
					}

					// normalizing the schedule, at this point without restriction on maximum dose, but the maximum doses can not be diminished
					NTD = 0.;
					for (int i = 1; i <= sizeRT; i++)
						NTD = NTD + RTscheme[1][i] + btoa*RTscheme[1][i] * RTscheme[1][i];

					nNorm = 1;
					while ((NTD > NTDMax + PossibleDamageError || NTD < NTDMax - PossibleDamageError) && (nNorm < nNormMax))
					{
						dr = NTDMax / NTD;

						NTD = 0.;

						for (int i = 1; i <= sizeRT; i++)
							if (RTscheme[1][i]<Dmax)
								RTscheme[1][i] = dr*RTscheme[1][i];

						for (int i = 1; i <= sizeRT; i++)
							NTD = NTD + RTscheme[1][i] + btoa*RTscheme[1][i] * RTscheme[1][i];

						nNorm++;
					}

					if (nNorm == nNormMax)
					{
						cout << "\nI can not normalize the scheme while searching for gradient -- the difference in tissue damage is " << NTD - NTDMax << "\n";
						system("PAUSE");
						exit(0);
					}

					// showing the schedule on screen
					Schedule_output();

					// running a simulation with this schedule
					SimulationRun();

					NTCminArray[sch_cnt] = NTCmin;

					//writing down the information about this simulation in a file
					fprintf(results_sch, "%i\t%i\t\t%.4e\t\n", grad_cnt, sch_cnt, NTCmin);
					fflush(results_sch);

					if (NTCminArray[sch_cnt] >= NTCminArray[0])
					{
						cout.precision(ss);
						cout << "Minimum tumor cell number is " << NTCminArray[sch_cnt] << " (+";
						cout.precision(5);
						cout << 100.*(NTCminArray[sch_cnt] - NTCminArray[0]) / NTCminArray[0] << "%)";
					}
					else
					{
						cout.precision(ss);
						cout << "Minimum tumor cell number is " << NTCminArray[sch_cnt] << " (";
						cout.precision(5);
						cout << 100.*(NTCminArray[sch_cnt] - NTCminArray[0]) / NTCminArray[0] << "%)";
					}

				}// end of the search for gradient (step 2)

				Nopt = 0;
				Kn = 1.;

				// step 3: moving down the gradient as long as it provides better schedule
				while (((NTCmin < (1. - 1.e-15)*NTCminOpt || Nopt == 0) && !((NTCmin < (1. - 1.e-15)*NTCminOpt) && (Kn < 1.)) && (StopS3 == 0)))
				{
					if (Kn == 1. && Nopt > 0)
						NTCminOpt = NTCmin;

					Nopt++;

					// creating the optimized schedule according to the found direction
					for (int i = 1; i <= sizeRT; i++)
						RTscheme[1][i] = max(0., RTscheme_opt[1][i] + Kn * Nopt * dD * (log(NTCminArray[0]) - log(NTCminArray[i])));

					// normalizing it
					NormalizeRTscheme();

					// setting counters
					t = t_preRT;
					ti = ti_preRT;
					tr = tr_preRT;
					Rad = Rad_preRT;
					doseRTcnt = 1;
					NTCmin = NTC_preRT;

					// setting distributions of variables
					for (int i = 0; i <= N; i++)
					{
					    n0[i] = n0preRT[i];
						m0[i] = m0preRT[i];
						g0[i] = g0preRT[i];
						w0[i] = w0preRT[i];

						n1[i] = n0preRT[i];
						m1[i] = m0preRT[i];
						g1[i] = g0preRT[i];
						w1[i] = w0preRT[i];
					}

					// showing it on screen
					cout.precision(ss);
					cout << "\n\nOptimized RT-scheme is:\n";
					for (int i = 1; i <= sizeRT; i++)
						cout << RTscheme[0][i] << "\t" << RTscheme[1][i] << "\n";
				
					// running a simulation with this schedule
					SimulationRun();

					if (Kn == 1.)
						cout << "Step 3: going down the gradient #" << Nopt << "\n";
					else
						cout << "Step 3: going down the gradient #" << Nopt << "; Kn is reduced and equal to " << Kn << "\n";

					if (NTCmin > NTCminOpt)
					{
						cout.precision(ss);
						cout << "Minimum tumor cell number is " << NTCmin << " (+";
						cout.precision(5);
						cout << 100.*(NTCmin - NTCminOpt) / NTCminOpt << "%)";
					}
					else
					{
						cout.precision(ss);
						cout << "Minimum tumor cell number is " << NTCmin << " (";
						cout.precision(5);
						cout << 100.*(NTCmin - NTCminOpt) / NTCminOpt << "%)";
					}

					// writing down the result, if it is optimized
					if (NTCmin < (1. - 1.e-15)*NTCminOpt)
					{
						fprintf(results_sch, "%i\tOpt-%i\t%.2f%%\t%.4e\n", grad_cnt, Nopt, 100.*(NTCmin - NTCminInit) / NTCminInit, NTCmin);
						fflush(results_sch);
						OptSchedule_output();
					}

					// If the treatment is not optimized during the first step down the gradient, trying to reduce the step
					if (NTCmin >= (1. - 1.e-15)*NTCminOpt && (Nopt == 1))
					{
						Kn = Kn / 2;
						if (Kn < knmin) // if the step is very small, exit
						{
							cout << "\nThe descent is finished\n";
							StopS3 = 1;
						}
						Nopt--;
					}

					//If there was an improvement under decreased step, optimized values should be rewritten here, since it will not enter this cycle again
					if (NTCmin < (1. - 1.e-15)*NTCminOpt && Kn < 1)
						NTCminOpt = NTCmin;
				}

				// When the descent is finished
				if (Kn == 1)
				{
					for (int i = 1; i <= sizeRT; i++)
						RTscheme[1][i] = max(0., RTscheme_opt[1][i] + Kn * (Nopt - 1) * dD *  (log(NTCminArray[0]) - log(NTCminArray[i])));

					NormalizeRTscheme();

					// rewrite the optimized scheme
					for (int i = 1; i <= sizeRT; i++)
					{
						RTscheme_opt[0][i] = RTscheme[0][i];
						RTscheme_opt[1][i] = RTscheme[1][i];
					}
				}
				else if (Kn > knmin)
				{
					for (int i = 1; i <= sizeRT; i++)
						RTscheme[1][i] = max(0., RTscheme_opt[1][i] + Kn * Nopt * dD *  (log(NTCminArray[0]) - log(NTCminArray[i])));

					NormalizeRTscheme();

					// rewrite the optimized scheme
					for (int i = 1; i <= sizeRT; i++)
					{
						RTscheme_opt[0][i] = RTscheme[0][i];
						RTscheme_opt[1][i] = RTscheme[1][i];
					}
				}

				NTCminArray[0] = NTCminOpt; // the actual scheme now corresponds to this cell number

				grad_cnt++;
				StopS3 = 0;
			}

			// step 4: the result can be further optimized by rearranging the doses of the second stage
			if (RTscheme_opt[1][sizeRT] < Dmax)
			{
				//determining the second stage
				isec = sizeRT;
				while (RTscheme_opt[1][isec - 1] > kfin*RTscheme_opt[1][sizeRT])
					isec--;

				NTDbeg = 0.;
				for (int i = 1; i <= isec - 1; i++)
					NTDbeg = NTDbeg + RTscheme[1][i] + btoa*RTscheme[1][i] * RTscheme[1][i];

				cout << "\nStep 4: trying to optimize the schedule, found by gradient descent\n";

				sizeRTsaved = sizeRT;
				int iSize = sizeRT;
				ef_sch_cnt = 1;
				EqualDoseCheckStop = 0;

				while (EqualDoseCheckStop == 0 && iSize > isec)
				{
					if (btoa == 0.)
						EqualDose = (NTDMax - NTDbeg) / (iSize - isec + 1);
					else
						EqualDose = (sqrt(1. + 4.*btoa*(NTDMax - NTDbeg) / (iSize - isec + 1)) - 1.) / (2.*btoa);

					if (EqualDose <= Dmax) // going to check the corresponding schedule only if the dose is less than maximum single dose
					{
						// setting counters
						t = t_preRT;
						tr = tr_preRT;
						Rad = Rad_preRT;
						doseRTcnt = 1;
						NTCmin = NTC_preRT;

						// setting distributions of variables
						for (int i = 0; i <= N; i++)
						{
							n0[i] = n0preRT[i];
							m0[i] = m0preRT[i];
							g0[i] = g0preRT[i];
							w0[i] = w0preRT[i];

							n1[i] = n0preRT[i];
							m1[i] = m0preRT[i];
							g1[i] = g0preRT[i];
							w1[i] = w0preRT[i];
						}

						// adjusting the schedule
						for (int i = isec; i <= iSize; i++)
							RTscheme[1][i] = EqualDose;

						sizeRT = iSize;

						if (iSize < sizeRT)
							for (int i = iSize + 1; i <= sizeRT; i++)
								RTscheme[1][i] = 0.;

						// showing the schedule on screen
						Schedule_S4_output();

						// running a simulation with this schedule
						SimulationRun();

						NTCminArray[ef_sch_cnt] = NTCmin;

						//writing down the information about this simulation in a file
						fprintf(results_sch, "S4\t%i\t\t%.4e\t\n", sizeRT, NTCmin);
						fflush(results_sch);
						
						if (NTCminArray[ef_sch_cnt] >= NTCminArray[0])
						{
							cout.precision(ss);
							cout << "Minimum tumor cell number is " << NTCminArray[ef_sch_cnt] << " (+";
							cout.precision(5);
							cout << 100.*(NTCminArray[ef_sch_cnt] - NTCminArray[0]) / NTCminArray[0] << "%)";
						}
						else
						{
							cout.precision(ss);
							cout << "Minimum tumor cell number is " << NTCminArray[ef_sch_cnt] << " (";
							cout.precision(5);
							cout << 100.*(NTCminArray[ef_sch_cnt] - NTCminArray[0]) / NTCminArray[0] << "%)";
						}

						iSize--;
						ef_sch_cnt++;
					}
					else
						EqualDoseCheckStop = 1;
				}

				ef_best = 1;
				for (int i = 2; i <= ef_sch_cnt - 1; i++)
					if (NTCminArray[i] < NTCminArray[ef_best])
						ef_best = i;

				//if the best of these schemes is worse (or equally good) than the previous, the latter will be final
				if (NTCminArray[ef_best] >= NTCminArray[0])
				{
					cout << "\n\nAll the schemes of step 4 are worse, than the previous one\n";
					cout << "\nI can not optimize no more\n";
					system("PAUSE");
					exit(0);
				}
				else // otherwise, it will be replaced by the most optimal
				{
					iSize = sizeRTsaved + 1 - ef_best;
					if (btoa == 0.)
						EqualDose = (NTDMax - NTDbeg) / (iSize - isec + 1);
					else
						EqualDose = (sqrt(1. + 4.*btoa*(NTDMax - NTDbeg) / (iSize - isec + 1)) - 1.) / (2.*btoa);
					cout << "\n\nStep 4 allowed to further optimize the scheme\n";

					for (int i = 1; i <= isec - 1; i++)
						RTscheme_opt[1][i] = RTscheme[1][i];

					for (int i = isec; i <= iSize; i++)
						RTscheme_opt[1][i] = EqualDose;

					sizeRT = iSize;
					if (iSize < sizeRT)
						for (int i = iSize + 1; i <= sizeRT; i++)
							RTscheme_opt[1][i] = 0.;

					sizeRT = min(sizeRT, iSize); // zero fractions will not be varied further
										
					NTCminArray[0] = NTCminArray[ef_best];
				}

				for (int i = 1; i <= sizeRT; i++)
					NTCminArray[i] = 0.;

				OptSchedule_output();
				grad_cnt++;

			}
			else // if the last dose is already Dmax
			{
				cout << "\nI can not optimize no more\n";
				system("PAUSE");
				exit(0);
			}
		}

	}

	End();
}

//***********************************************************************************************************************************************************
//********************************* BEGINNING OF SIMULATION *************************************************************************************************
//***********************************************************************************************************************************************************

void Begin()
{
	srand(time(NULL));

	//********* READING RT SCHEDULE

	// determining the number of fractions

	if ((inputRT = fopen("InitialScheme.txt", "rt")) != NULL)
	{
		int fraction_counter = 0;
		double one, two;
		char * word;
		word = new char[256];
		do {
			fgets(word, 256, inputRT);
			int iScan = sscanf(word, "%lf", &one);
			if (iScan == 1)
				++fraction_counter;
			strcpy(word, "qwerty");
		} while (!feof(inputRT));

		sizeRT = fraction_counter;
		NTCminArray = new double[sizeRT + 1];
		cout << "\nsizeRT=" << sizeRT << "\n";

		RTscheme = new double*[2]; // 0 -- time of fraction, 1 -- dose
		RTscheme_opt = new double*[2]; // the first current optimal scheme is the initial scheme
		RTeff_dose = new double**[2]; // 0 -- alpha, 1 -- beta

		for (int i = 0; i <= 1; i++)
		{
			RTscheme[i] = new double[sizeRT + 1]; // 0-th element will be ignored for convenience
			RTscheme_opt[i] = new double[sizeRT + 1];
		}

		for (int i = 0; i <= 1; i++)
			RTeff_dose[i] = new double*[N + 1];

		for (int j = 0; j <= 1; j++)
			for (int i = 0; i <= N; i++)
				RTeff_dose[j][i] = new double[sizeRT + 1];

		// creating RT scheme

		rewind(inputRT);

		fraction_counter = 0;
		do
		{
			fgets(word, 256, inputRT);
			int iScan = sscanf(word, "%lf%lf", &one, &two);

			RTscheme[0][0] = 0.; // they will be ignored
			RTscheme_opt[0][0] = 0.;

			if (iScan == 2)
			{
				RTscheme[0][fraction_counter + 1] = 1.*one;
				RTscheme[1][fraction_counter + 1] = 1.*two;
				NTDMax = NTDMax + RTscheme[1][fraction_counter + 1] + btoa*RTscheme[1][fraction_counter + 1] * RTscheme[1][fraction_counter + 1];
				RTscheme_opt[0][fraction_counter + 1] = 1.*one;
				RTscheme_opt[1][fraction_counter + 1] = 1.*two;
				++fraction_counter;
			}
			strcpy(word, "qwerty");
		} while (!feof(inputRT));

		fclose(inputRT);

		cout << "\nRT scheme is:\n";

		for (int i = 1; i <= sizeRT; i++)
			cout << RTscheme[0][i] << "\t" << RTscheme[1][i] << "\n";

		cout << "\n";
		cout << "Total relative tissue damage is " << NTDMax << "\n";

	}
	else
	{
		if (SoloRun == 1)
		{
			cout << "There is no radiotherapy schedule, simulating untreated growth\n";
			sizeRT = 42;
			RTscheme = new double*[2];
			for (int i = 0; i <= 1; i++)
				RTscheme[i] = new double[sizeRT + 1];
			for (int i = 1; i <= sizeRT; i++)
			{
				RTscheme[0][i] = 24.*(i - 1);
				RTscheme[1][i] = 0.; // effectively no treatment
			}
			RTeff_dose = new double**[2]; // 0 -- alpha, 1 -- beta

			for (int i = 0; i <= 1; i++)
				RTeff_dose[i] = new double*[N + 1];

			for (int j = 0; j <= 1; j++)
				for (int i = 0; i <= N; i++)
					RTeff_dose[j][i] = new double[sizeRT + 1];
		}
		else
		{
			// if no initial scheme is suggested, then standard scheme will be used as the reference one

			sizeRT = 42; // 6 weeks
			NTCminArray = new double[sizeRT + 1];

			cout << "\nsizeRT=" << sizeRT << "\n";

			RTscheme = new double*[2]; // 0 -- time of fraction, 1 -- dose
			RTscheme_opt = new double*[2]; // the first current optimal scheme is the initial scheme
			RTeff_dose = new double**[2]; // 0 -- alpha, 1 -- beta

			for (int i = 0; i <= 1; i++)
			{
				RTscheme[i] = new double[sizeRT + 1]; // 0-th element will be ignored for convenience
				RTscheme_opt[i] = new double[sizeRT + 1];
			}
			
			for (int i = 0; i <= 1; i++)
				RTeff_dose[i] = new double*[N + 1];

			for (int j = 0; j <= 1; j++)
				for (int i = 0; i <= N; i++)
					RTeff_dose[j][i] = new double[sizeRT + 1];

			RTscheme[0][0] = 0.; // they will be ignored
			RTscheme_opt[0][0] = 0.;

			for (int i = 1; i <= sizeRT; i++)
			{
				RTscheme[0][i] = 24.*(i - 1);
				RTscheme[1][i] = 2.; // further weekends will be removed
			}

			// removing weekends
			RTscheme[1][6] = 0.;
			RTscheme[1][7] = 0.;
			RTscheme[1][13] = 0.;
			RTscheme[1][14] = 0.;
			RTscheme[1][20] = 0.;
			RTscheme[1][21] = 0.;
			RTscheme[1][27] = 0.;
			RTscheme[1][28] = 0.;
			RTscheme[1][34] = 0.;
			RTscheme[1][35] = 0.;
			RTscheme[1][41] = 0.;
			RTscheme[1][42] = 0.;

			for (int i = 1; i <= sizeRT; i++)
			{
				RTscheme_opt[0][i] = RTscheme[0][i];
				RTscheme_opt[1][i] = RTscheme[1][i];
				NTDMax = NTDMax + RTscheme[1][i] + btoa*RTscheme[1][i] * RTscheme[1][i];
			}

			cout << "\nRT scheme is:\n";

			for (int i = 1; i <= sizeRT; i++)
				cout << RTscheme[0][i] << "\t" << RTscheme[1][i] << "\n";

			cout << "\n";
			cout << "Total relative tissue damage is " << NTDMax << "\n";

		}
	}

	for (int i = 0; i <= NwS; i++) // in order for initial oxygen level to be calculated under any inflow
		Sfix[i] = S(wLimS*i / NwS);

	//********* INITIAL CONDITIONS

	for (int i = 0; i <= N; i++)
	{
		n0[i] = 0.;
		m0[i] = 0.;
		g0[i] = 1.; // will be recalculated right now
		w0[i] = 2.; // will be recalculated right now
	}

	for (int i = 0; i <= inception_width_int; i++)
		n0[i] = 0.1;

	for (int i = 0; i <= N; i++)
	{
		n1[i] = n0[i];
		m1[i] = m0[i];
		g1[i] = g0[i];
		w1[i] = w0[i];
	}

	Oxygen_Glucose_qs(); // nutrients values are replaced with quasistationary ones
	winit = w0[N];

	Radius_CellNumber_Speed_O2_estimation();

}

//***********************************************************************************************************************************************************
//********************************* OUTPUT OF DISTRIBUTIONS OF VARIABLES ************************************************************************************
//***********************************************************************************************************************************************************

void Profile_output()
{
	//********* CREATING A FILENAME WITH THE SERIAL NUMBER

	d = (int)((ti + 0.00001) / output_frequency);
	_itoa(d, num, 10);

	if (nopo == 0)
	{
		if (d < 10)
			strcpy(File_name1, "res-d000"); // designed for maximum a thousand of files
		else if (d < 100)
			strcpy(File_name1, "res-d00");
		else if (d < 1000)
			strcpy(File_name1, "res-d0");
		else
			strcpy(File_name1, "res-d");

		strcat(File_name1, num);
		strcat(File_name1, ".txt");

		//********* OUTPUT OF DISTRIBUTIONS OF VARIABLES INTO A FILE

		FILE * out;
		if ((out = fopen(File_name1, "wt")) == NULL)
			fprintf(stderr, "Cannot open file\n");

		fprintf(out, "Len(mm) n               m               g               w               I	\n\n");

		for (int i = 0; i <= N; i++)
			fprintf(out, "%.3f\t%.7e\t%.7e\t%.7e\t%.7e\t%.7e\n", i*h / 10., n1[i], m1[i], g1[i], w1[i], I0[i]);
		fclose(out);
	}

	//********* PRINTING THE SERIAL NUMBER, RADIUS, SPEED OF TUMOR FRONT AND NUMBER OF TUMOR CELLS ON A SCREEN

	printf("day %i, radius = %.3f mm, speed = %.3f mm/week, cell number = %.3e\n", d, Rad, V, NTC);

	//********* SWITCHING THE COUNTER FOR OUTPUT OF DISTRIBUTION OF VARIABLES 

	ti = ti + output_frequency;
}

//***********************************************************************************************************************************************************
//********************************* ESTIMATION OF RADIUS, SPEED, ETC. ******************************************************************************************
//***********************************************************************************************************************************************************

void Radius_CellNumber_Speed_O2_estimation()
{
	Radius_Speed_estimation();

	//********* COUNTING TUMOR CELLS AND ESTIMATING OXYGEN LEVEL
	NTC = 0.;
	NPTC = 0.;
	O2 = 0.;

	for (int i = 0; i <= N; i++)
	{
		nTC = 4.*3.14* ((1.*i + 0.5)*h)* ((1.*i + 0.5)*h)* h *300.* n1[i];
		NTC = NTC + nTC;
		NPTC = NPTC + nTC * g1[i] / (g1[i] + gcr);
		O2 = O2 + w1[i] * nTC;
	}
	O2 = 17.024*O2 / NTC; // in mmHg

	if (NTC < NTCmin)
		NTCmin = NTC;
}

void Radius_Speed_estimation()
{
	Radius_estimation();

	//********* ESTIMATION OF TUMOR FRONT SPEED

	if (abs(Rad - Rad_prev)>0.1 && Rad > inception_width_mm + 0.1)
	{
		V = 7.*24.*(Rad - Rad_prev) / (t - T_prev); // in mm/week
		T_prev = t;
		Rad_prev = Rad;
	}
}

void Radius_estimation()
{
	//********* ESTIMATION OF RADIUS
	Rad = 0.;

	for (int i = 0; i <= N - 1; i++)
		if ((n1[i] + m1[i] >= nR) && (n1[i + 1] + m1[i + 1]<nR))
			Rad = 1.*i*h / 10.;
}

void CellNumber_estimation()
{
	//********* COUNTING TUMOR CELLS
	NTC = 0.;
	NPTC = 0.;

	for (int i = 1; i <= N; i++) // 0-th element is ignored -- there should always be necrotic core in a large tumor
	{
		nTC = 4.*3.14* ((1.*i + 0.5)*h)* ((1.*i + 0.5)*h)* h *300.* n1[i];
		NTC = NTC + nTC;
		NPTC = NPTC + nTC * g1[i] / (g1[i] + gcr);
	}

	if (NTC < NTCmin)
		NTCmin = NTC;
}
//***********************************************************************************************************************************************************
//********************************* OUTPUT OF THE TUMOR RADIUS, SPEED OF TUMOR FRONT AND NUMBER OF TUMOR CELLS **********************************************
//***********************************************************************************************************************************************************

void Radius_Speed_CellNumber_output()
{
	tr = tr + rsn_frequency;

	//********* OUTPUT

	fprintf(results, "%.1f\t%.5f\t%.5f\t%.5e\t%.5e\t%.5f\t%.5f\n", t, Rad, V, NTC, NPTC, O2, 17.024*w0[0]);
	fflush(results);
}

//***********************************************************************************************************************************************************
//********************************* SOLUTION OF KINETIC TERMS ******************************************************************************************
//***********************************************************************************************************************************************************

//********* function that defines the rate of cell death depending on the level of oxygen

double Mfunc(double ww)
{
	if (ww < wcr)
		return (M*((ww / wcr)*(ww / wcr) - 2.*(ww / wcr) + 1.));
	else
		return 0.;
}

//********* function that defines the rate of oxygen inflow depending on the level of oxygen

double S(double ww)
{
	return (pow(ww, chi - 1) / (pow(wh, chi) + pow(ww, chi))); // first exponent is modified for the quasi-stationary solution
}

//********* KINETIC FUNCTIONS

double Growthn(double nn, double gg, double mw) // for tumor cells
{
	return(B*nn*gg / (gg + gcr) - eps*mw*nn);
}

double Growthm(double nn, double mm, double mw) // for necrotic tissue
{
	return (eps*mw*nn + (mw + M*nn)*(1. - nn - mm));
}

//********* EULER PROCEDURE

void Euler()
{
	// calculation of reaction and convection terms in the region, where n+m=0, is skipped
	ConvRad = N;

	for (int i = ConvRad; i <= N - 1; i++)
		if ((nd[i] + mr[i] > 0.) && (nd[i + 1] + mr[i + 1] == 0.))
			ConvRad = i + 5; // + 5 -- just in case

	for (int i = 0; i <= ConvRad; i++)
	{
		Mw[i] = Mfunc(w0[i]);

		nr[i] = n0[i] + tau*Growthn(n0[i], g0[i], Mw[i]);
		mr[i] = m0[i] + tau*Growthm(n0[i], m0[i], Mw[i]);
	}
}

//********* QUASI-STATIONARY APPROXIMATIONS FOR NUTRIENTS

void Oxygen_Glucose_qs()
{
	for (int i = 0; i <= N; i++)
	{
		h1[i] = max(0., 1. - n1[i] - m1[i]); // to overcome numerical errors
		G0[i] = g0[i] * (1.*i + 0.5)*h; // for easy solution in spherical symmetrical case
		W0[i] = w0[i] * (1.*i + 0.5)*h;
	}

	//Solution for glucose

	norm_g = 1.;

	for (int i = 0; i <= N; i++)
	{
		Fg[i] = Pg*h1[i] * (1.*i + 0.5)*h / kg;
		alpha_g[i] = (Qgn*n1[i] + Qgh*h1[i]) / kg; // in order to not recalculate it a lot of times
	}

	kqs = 1;

	while ((norm_g > norm_min) && (kqs < k_crit))
	{
		for (int i = 0; i <= N; i++)
			bprg[i] = 2. + Pg*h1[i] / kg + alpha_g[i] / (G0[i] / ((1.*i + 0.5)*h) + gcr);

		muleftg = 0.;
		hileftg = 0.5 / 1.5;

		murightg = 0.;
		hirightg = (1.*N + 0.5) / (1.*N - 0.5);

		TMA_variable(aprg, bprg, cprg, Fg, progg, muleftg, hileftg, murightg, hirightg);

		for (int i = 0; i <= N; i++)
			G1[i] = progg[i];

		norm_g = 0.;

		for (int i = 0; i <= N; i++)
		{
			norm_g = max(norm_g, abs(G0[i] - G1[i]));
			G0[i] = G1[i];
		}

		kqs++;
	}
	
	for (int i = 0; i <= N; i++)
	{
		g0[i] = G0[i] / ((1.*i + 0.5)*h);
		g1[i] = G1[i] / ((1.*i + 0.5)*h);
	}

	//Solution for oxygen, it depends on glucose

	norm_w = 1.;

	for (int i = 0; i <= N; i++)
	{
		alpha_w[i] = ((Qwn*g1[i] / (g1[i] + gcr) + Qwh*gcr / (g1[i] + gcr))*n1[i] + Qwh*h1[i]) / kw; // in order to not recalculate it a lot of times
		beta_w[i] = Pw*h1[i] * (1.*i + 0.5)*h / kw;
		Fw[i] = beta_w[i] * SwA;
	}

	kqs = 1;

	while ((norm_w > norm_min) && (kqs < k_crit))
	{
		for (int i = 0; i <= N; i++)
			bprw[i] = 2. + alpha_w[i] / ((W0[i] / ((1.*i + 0.5)*h)) + wcr) + (Pw*h1[i] / kw) * Sfix[(int)((((W0[i] / ((1.*i + 0.5)*h)) / wLimS) + NmdS) *NwS)];

		muleftw = 0.;
		hileftw = 0.5 / 1.5;

		murightw = 0.;
		hirightw = (1.*N + 0.5) / (1.*N - 0.5);

		TMA_variable(aprw, bprw, cprw, Fw, progw, muleftw, hileftw, murightw, hirightw);

		for (int i = 0; i <= N; i++)
			W1[i] = progw[i];

		norm_w = 0.;

		for (int i = 0; i <= N; i++)
		{
			norm_w = max(norm_w, abs(W0[i] - W1[i]));
			W0[i] = W1[i];
		}

		kqs++;
	}
	
	for (int i = 0; i <= N; i++)
	{
		w0[i] = W0[i] / ((1.*i + 0.5)*h);
		w1[i] = W1[i] / ((1.*i + 0.5)*h);
	}
}

//***********************************************************************************************************************************************************
//********************************* MOTION OF CELLS *********************************************************************************************************
//***********************************************************************************************************************************************************

//********* PROCEDURE FOR CELL MIGRATION 

void Migration_cells()
{
	for (int i = 0; i <= N; i++)
	{
		N0[i] = n0[i] * (1.*i + 0.5)*h; // for easy solution in spherical symmetrical case
		NR[i] = nr[i] * (1.*i + 0.5)*h;
	}

	for (int i = 1; i <= N - 1; i++)
		Fn[i] = (N0[i + 1] - 2.*N0[i] + N0[i - 1]) + kn * NR[i];

	muleftn = 0.;
	hileftn = 0.5 / 1.5;

	murightn = 0.;
	hirightn = (1.*N + 0.5) / (1.*N - 0.5);

	TMA(aprn, bprn, cprn, Fn, progn, muleftn, hileftn, murightn, hirightn);

	for (int i = 0; i <= N; i++)
		ND[i] = progn[i];

	for (int i = 0; i <= N; i++)
		nd[i] = ND[i] / ((1.*i + 0.5)*h);
}

//********* PROCEDURE FOR COMPUTING CONVECTIVE SPEED

void ConvSpeed(double* nn, double* gg, double* FF, double* II)
{
	FF[0] = (B * nn[0] * gg[0] / (gg[0] + gcr)) + Dn*((2. / (0.5*h))*((nn[1] - nn[0]) / (2.*h)) + ((nn[1] - 2.*nn[0] + nn[0]) / (2.*h*h)));

	for (int i = 1; i <= ConvRad + 1; i++)
		FF[i] = (B * nn[i] * gg[i] / (gg[i] + gcr)) + Dn*((2. / ((1.*i + 0.5)*h))*((nn[i + 1] - nn[i - 1]) / (2.*h)) + ((nn[i + 1] - 2.*nn[i] + nn[i - 1]) / (2.*h*h)));

	for (int i = 0; i <= ConvRad + 1; i++)
		II[i] = II[i - 1] + h*(1.*i + 0.5)*h*(1.*i + 0.5)*h*FF[i];

	for (int i = 0; i <= ConvRad + 1; i++)
		II[i] = II[i] / ((1.*i + 0.5)*h*(1.*i + 0.5)*h);
}

//********* PROCEDURE FOR CELLS CONVECTION TERM

void Convection()
{
	for (int i = 0; i <= ConvRad; i++)
	{
		NDC[i] = nd[i] * ((1.*i + 0.5)*h*(1.*i + 0.5)*h); // for easy solution in spherical symmetrical case
		MRC[i] = mr[i] * ((1.*i + 0.5)*h*(1.*i + 0.5)*h);
	}

	ConvSpeed(n0, g0, F0, I0_not_corr);

	if (I0_not_corr[0] >= 0.)
		I0[0] = (1. - (I0_not_corr[0] * tau / (2.*h)))*I0_not_corr[0] + (I0_not_corr[0] * tau / (2.*h))*I0_not_corr[1];
	else
		I0[0] = (1. + (I0_not_corr[0] * tau / (2.*h)))*I0_not_corr[0] + (I0_not_corr[0] * tau / (2.*h))*I0_not_corr[0];

	for (int i = 1; i <= ConvRad + 1; i++)
	{
		if (I0_not_corr[i] >= 0.)
			I0[i] = (1. - (I0_not_corr[i] * tau / (2.*h)))*I0_not_corr[i] + (I0_not_corr[i] * tau / (2.*h))*I0_not_corr[i + 1];
		else
			I0[i] = (1. + (I0_not_corr[i] * tau / (2.*h)))*I0_not_corr[i] - (I0_not_corr[i] * tau / (2.*h))*I0_not_corr[i - 1];
	}

	for (int i = 0; i <= ConvRad; i++)
		Q_plus[i] = (0.5 - I0[i] * tau / h) / (1. + (I0[i + 1] - I0[i])*tau / h);
	for (int i = 1; i <= ConvRad; i++)
		Q_minus[i] = (0.5 + I0[i] * tau / h) / (1. - (I0[i - 1] - I0[i])*tau / h);

	N1C[0] = 0.5*Q_plus[0] * Q_plus[0] * (NDC[1] - NDC[0]) + (Q_plus[0])*NDC[0] + (0.5*h*NDC[0]) / (0.5*h + I0[0] * tau);
	M1C[0] = 0.5*Q_plus[0] * Q_plus[0] * (MRC[1] - MRC[0]) + (Q_plus[0])*MRC[0] + (0.5*h*MRC[0]) / (0.5*h + I0[0] * tau);

	for (int i = 1; i <= ConvRad - 1; i++)
	{
		N1C[i] = 0.5*Q_minus[i] * Q_minus[i] * (NDC[i - 1] - NDC[i]) + 0.5*Q_plus[i] * Q_plus[i] * (NDC[i + 1] - NDC[i]) + (Q_plus[i] + Q_minus[i])*NDC[i];
		M1C[i] = 0.5*Q_minus[i] * Q_minus[i] * (MRC[i - 1] - MRC[i]) + 0.5*Q_plus[i] * Q_plus[i] * (MRC[i + 1] - MRC[i]) + (Q_plus[i] + Q_minus[i])*MRC[i];
	}

	N1C[ConvRad] = 0.5*Q_minus[ConvRad] * Q_minus[ConvRad] * (NDC[ConvRad - 1] - NDC[ConvRad]) - 0.5*Q_plus[ConvRad] * Q_plus[ConvRad] * NDC[ConvRad] + (Q_plus[ConvRad] + Q_minus[ConvRad])*NDC[ConvRad];
	M1C[ConvRad] = 0.5*Q_minus[ConvRad] * Q_minus[ConvRad] * (MRC[ConvRad - 1] - MRC[ConvRad]) - 0.5*Q_plus[ConvRad] * Q_plus[ConvRad] * MRC[ConvRad] + (Q_plus[ConvRad] + Q_minus[ConvRad])*MRC[ConvRad];

	//********* ANTIDIFFUSION STAGE

	// n
	for (int i = 1; i <= N - 1; i++)
		Fn[i] = -N1C[i];

	muleftn = 0.;
	hileftn = 0.5*0.5 / (1.5*1.5);

	murightn = 0.;
	hirightn = (1.*N + 0.5)*(1.*N + 0.5) / ((1.*N - 0.5)*(1.*N - 0.5));

	TMA((1. / 8.), (-3. / 4.), (1. / 8.), Fn, progn, muleftn, hileftn, murightn, hirightn);

	for (int i = 0; i <= N; i++)
		N1C[i] = progn[i];

	// m
	for (int i = 1; i <= N - 1; i++)
		Fn[i] = -M1C[i];

	muleftm = 0.;
	hileftm = (0.5)*(0.5) / ((1.5)*(1.5));

	murightm = 0.;
	hirightm = (1.*N + 0.5)*(1.*N + 0.5) / ((1.*N - 0.5)*(1.*N - 0.5));

	TMA((1. / 8.), (-3. / 4.), (1. / 8.), Fn, progm, muleftm, hileftm, murightm, hirightm);

	for (int i = 0; i <= N; i++)
		M1C[i] = progm[i];

	// REDEFINITION
	for (int i = 0; i <= ConvRad; i++)
	{
		n1[i] = N1C[i] / ((1.*i + 0.5)*h*(1.*i + 0.5)*h);
		m1[i] = M1C[i] / ((1.*i + 0.5)*h*(1.*i + 0.5)*h);
	}

	for (int i = ConvRad + 1; i <= N; i++)
	{
		n1[i] = 0.;
		m1[i] = 0.;
	}
}

//********* TRIDIAGONAL MATRIX ALGORITHM

void TMA(double apr, double bpr, double cpr, double* F, double* prog,
	double muleft, double hileft, double muright, double hiright)
{
	pp[1] = hileft;
	qq[1] = muleft;

	for (int i = 1; i <= N - 1; i++)
	{
		pp[i + 1] = cpr / (bpr - pp[i] * apr);
		qq[i + 1] = (apr*qq[i] + F[i]) / (bpr - pp[i] * apr);
	}

	prog[N] = (muright + hiright*qq[N]) / (1. - pp[N] * hiright);

	for (int i = N - 1; i >= 0; i--)
		prog[i] = pp[i + 1] * prog[i + 1] + qq[i + 1];
}

void TMA_variable(double apr, double* bpr, double cpr, double* F, double* prog,
	double muleft, double hileft, double muright, double hiright)
{
	pp[1] = hileft;
	qq[1] = muleft;

	for (int i = 1; i <= N - 1; i++)
	{
		pp[i + 1] = cpr / (bpr[i] - pp[i] * apr);
		qq[i + 1] = (apr*qq[i] + F[i]) / (bpr[i] - pp[i] * apr);
	}

	prog[N] = (muright + hiright*qq[N]) / (1. - pp[N] * hiright);

	for (int i = N - 1; i >= 0; i--)
		prog[i] = pp[i + 1] * prog[i + 1] + qq[i + 1];
}

//***********************************************************************************************************************************************************
//********************************* RADIOTHERAPY ************************************************************************************************************
//***********************************************************************************************************************************************************

void RT()
{
	for (int i = 0; i <= N; i++)
	{
		RTeff_dose[0][i][doseRTcnt] = RTscheme[1][doseRTcnt] * ((g0[i] + k*gcr) / (g0[i] + gcr)) * ((w0[i] * OERaM + Km) / (w0[i] + Km));
		RTeff_dose[1][i][doseRTcnt] = RTscheme[1][doseRTcnt] * ((g0[i] + k*gcr) / (g0[i] + gcr)) * ((w0[i] * OERbM + Km) / (w0[i] + Km));
	}

	for (int i = 0; i <= N; i++)
		RTsurv[i] = exp((-alpha*RTeff_dose[0][i][doseRTcnt] - beta*RTeff_dose[1][i][doseRTcnt] * RTeff_dose[1][i][doseRTcnt]));

	for (int i = 0; i <= N; i++)
	{
		m0[i] = m0[i] + n0[i] * (1. - RTsurv[i]);
		n0[i] = n0[i] * RTsurv[i];
	}
}

//***********************************************************************************************************************************************************
//********************************* MAIN CYCLE ENDING *******************************************************************************************************
//***********************************************************************************************************************************************************

void Cycle_Ending()
{
	for (int i = 0; i <= N; i++)
	{
		n0[i] = max(0., n1[i]);
		m0[i] = max(0., m1[i]);
		g0[i] = g1[i];
		w0[i] = w1[i];
	}
}

//***********************************************************************************************************************************************************
//********************************* OPTIMIZATION -- WRITING DOWN CURRENT SCHEDULE WITHIN CHECKING UNIFORMLY FRACTIONATED SCHEMES (STEP 1) *******************
//***********************************************************************************************************************************************************

void Schedule_EF_output()
{
	cout.precision(ss);
	cout << "\n\nUniformly fractionated schedule #" << ef_sch_cnt << ":\n";

	for (int i = 1; i <= sizeRT; i++)
		cout << RTscheme[0][i] << "\t" << RTscheme[1][i] << "\n";

	strcpy(File_name_sch, "EF-RTscheme-");

	_gcvt(ef_sch_cnt, 10, number);
	strcat(File_name_sch, number);

	strcat(File_name_sch, "txt");

	FILE * out_sch;
	if ((out_sch = fopen(File_name_sch, "wt")) == NULL)
		fprintf(stderr, "Cannot open file\n");

	for (int i = 1; i <= sizeRT; i++)
		fprintf(out_sch, "%.2f\t%.6f\n",
			RTscheme[0][i], RTscheme[1][i]);

	fclose(out_sch);
}

//***********************************************************************************************************************************************************
//********************************* OPTIMIZATION -- WRITING DOWN CURRENT SCHEDULE WITHIN A SEARCH FOR GRADIENT (step 2) *************************************
//***********************************************************************************************************************************************************

void Schedule_output()
{
	cout.precision(ss);
	cout << "\n\nSchedule #" << sch_cnt << ":\n";

	for (int i = 1; i <= sizeRT; i++)
		cout << RTscheme[0][i] << "\t" << RTscheme[1][i] << "\n";

	strcpy(File_name_sch, "RTscheme-");
	_itoa(grad_cnt, number, 10);
	strcat(File_name_sch, number);

	strcat(File_name_sch, "-");

	_gcvt(sch_cnt, 10, number);
	strcat(File_name_sch, number);

	strcat(File_name_sch, "txt");

	FILE * out_sch;
	if ((out_sch = fopen(File_name_sch, "wt")) == NULL)
		fprintf(stderr, "Cannot open file\n");

	for (int i = 1; i <= sizeRT; i++)
		fprintf(out_sch, "%.2f\t%.6f\n",
			RTscheme[0][i], RTscheme[1][i]);

	fclose(out_sch);
}

//***********************************************************************************************************************************************************
//********************************* OPTIMIZATION -- WRITING DOWN OPTIMIZED SCHEDULE WHILE GOING DOWN THE GRADIENT (step 3) **********************************
//***********************************************************************************************************************************************************

void OptSchedule_output()
{
	//cout << "\n";

	strcpy(File_name_sch, "OptRTscheme-");
	_itoa(grad_cnt, number, 10);

	if (grad_cnt < 10)
	{
		strcat(File_name_sch, "0");
		strcat(File_name_sch, number);
	}
	else
		strcat(File_name_sch, number);

	strcat(File_name_sch, "-");
	_itoa(Nopt, number, 10);

	if (Nopt < 10)
	{
		strcat(File_name_sch, "0");
		strcat(File_name_sch, number);
	}
	else
		strcat(File_name_sch, number);

	strcat(File_name_sch, ".txt");

	FILE * out_sch;
	if ((out_sch = fopen(File_name_sch, "wt")) == NULL)
		fprintf(stderr, "Cannot open file\n");

	for (int i = 1; i <= sizeRT; i++)
		fprintf(out_sch, "%.2f\t%.6f\n",
			RTscheme[0][i], RTscheme[1][i]);

	fclose(out_sch);
}

//***********************************************************************************************************************************************************
//********************************* OPTIMIZATION -- WRITING DOWN CURRENT SCHEDULE WITHIN STEP 4 *************************************************************
//***********************************************************************************************************************************************************

void Schedule_S4_output()
{
	cout.precision(ss);
	cout << "\n\nStep 4 schedule #" << ef_sch_cnt << ":\n";

	for (int i = 1; i <= sizeRT; i++)
		cout << RTscheme[0][i] << "\t" << RTscheme[1][i] << "\n";

	strcpy(File_name_sch, "S4-");

	_itoa(grad_cnt, number, 10);

	if (grad_cnt < 10)
	{
		strcat(File_name_sch, "0");
		strcat(File_name_sch, number);
	}
	else
		strcat(File_name_sch, number);

	strcat(File_name_sch, "-RTscheme-");

	_gcvt(ef_sch_cnt, 10, number);
	strcat(File_name_sch, number);

	strcat(File_name_sch, "txt");

	FILE * out_sch;
	if ((out_sch = fopen(File_name_sch, "wt")) == NULL)
		fprintf(stderr, "Cannot open file\n");

	for (int i = 1; i <= sizeRT; i++)
		fprintf(out_sch, "%.2f\t%.6f\n",
			RTscheme[0][i], RTscheme[1][i]);

	fclose(out_sch);
}

//***********************************************************************************************************************************************************
//********************************* NORMALIZATION OF THE SCHEME WITH ACCOUNT OF MAXIMUM FRACTIONAL DOSE *****************************************************
//***********************************************************************************************************************************************************

void NormalizeRTscheme()
{
	NTD = 0.;
	for (int i = 1; i <= sizeRT; i++)
		NTD = NTD + RTscheme[1][i] + btoa*RTscheme[1][i] * RTscheme[1][i];

	nNorm = 1;
	while ((NTD > NTDMax + PossibleDamageError || NTD < NTDMax - PossibleDamageError) && (nNorm < nNormMax))
	{
		dr = NTDMax / NTD;

		NTD = 0.;

		for (int i = 1; i <= sizeRT; i++)
			if (RTscheme[1][i] != Dmax)
				RTscheme[1][i] = min(Dmax, dr*RTscheme[1][i]);

		for (int i = 1; i <= sizeRT; i++)
			NTD = NTD + RTscheme[1][i] + btoa*RTscheme[1][i] * RTscheme[1][i];

		nNorm++;
	}

	if (nNorm == nNormMax)
	{
		cout << "\nI can not normalize the scheme -- the difference in tissue damage is " << NTD - NTDMax << "\n";
		system("PAUSE");
		exit(0);
	}
}

//***********************************************************************************************************************************************************
//********************************* OPTIMIZATION -- MAKE A SIMULATION WITH A SCHEDULE ******************************************
//***********************************************************************************************************************************************************

void SimulationRun()
{
	// the program runs until the last irradiation is performed -- minimum number of tumor cells is achieved at that moment
	while (doseRTcnt <= sizeRT)
	{
		t = t + tau;

		if (t - t_preRT - RTscheme[0][doseRTcnt] >= 0 && doseRTcnt <= sizeRT)
		{
			RT();
			doseRTcnt++;
		}

		Euler();

		if (Dn == 0.)
			for (int i = 0; i <= N; i++)
				nd[i] = nr[i];
		else
			Migration_cells();

		Convection();
		Oxygen_Glucose_qs();

		Cycle_Ending();

		if (t >= tr)
		{
			Radius_estimation();
			CellNumber_estimation();
		}
	}
}

//***********************************************************************************************************************************************************
//********************************* PROGRAM ENDING **********************************************************************************************************
//***********************************************************************************************************************************************************

void End()
{
	if (clock() / CLOCKS_PER_SEC >= 3600)
		printf("\n%s%ld%s%ld%s%ld%s", "The end. Total time: ", (clock() / CLOCKS_PER_SEC) / 3600, " h ", (clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600), " min ", (clock() / CLOCKS_PER_SEC) - 3600 * ((clock() / CLOCKS_PER_SEC) / 3600) - 60 * ((clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600)), " s.");
	else if (clock() / CLOCKS_PER_SEC >= 60)
		printf("\n%s%ld%s%ld%s", "The end. Total time: ", (clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600), " min ", (clock() / CLOCKS_PER_SEC) - 3600 * ((clock() / CLOCKS_PER_SEC) / 3600) - 60 * ((clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600)), " s.");
	else
		printf("\n%s%ld%s", "The end. Total time: ", (clock() / CLOCKS_PER_SEC) - 3600 * ((clock() / CLOCKS_PER_SEC) / 3600) - 60 * ((clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600)), " s.");
	_getch();
}

//********************************************************
//********* THE END
//********************************************************
